package Email::List;

use strict;
use Email::List::Subscription;

sub new
{
    my $class = shift(@_);
    my $listname = shift(@_);

    return bless({ listname => $listname, subscriptions => [] }, $class);
}

sub add_subscription
{
    my $self = shift(@_);
    my $subscription = shift(@_);

    push(@{$self->{subscriptions}}, $subscription);
    return 1;
}

sub remove_subscription
{
    my $self = shift(@_);
    my $subscriber = shift(@_);
    my $date = shift(@_);

# TODO: Implement list search & removal.
#    $self->{subscriptions}
#      = new Email::List::Subscription($self->{listname},
#				      $subscriber,
#				      $Email::Subscription::UNSUBSCRIPTION,
#				      $date);

    return 1;
}

sub sort_subscriptions
{
    my $self = shift(@_);

    @{$self->{subscriptions}}
      = sort Email::List::Subscription::cmp_date @{$self->{subscriptions}};

    return 1;
}

sub prune_subscriptions
{
    my $self = shift(@_);
    my $subs_ref = $self->{subscriptions};
    my $i;

    # Sort all the subscriptions into chronological order.
    $self->sort_subscriptions();

    # Remove any subscription-unsubscription pairs.
    for ($i = 0; $i <= $#{$subs_ref};)
    {
	# Remove unpaired unsubscriptions.
	if (   $subs_ref->[$i]->{type}
	    == $Email::List::Subscription::UNSUBSCRIPTION)
	{
	    splice(@{$subs_ref}, $i, 1);
	    next;
	}
	# Pair up a subscription with an unsubscription and then remove
	# them both.
	elsif (   $subs_ref->[$i]->{type}
	    == $Email::List::Subscription::SUBSCRIPTION)
	{
	    my $matched = 0;
	    my $j;

	    for ($j = 0; $j <= $#{$subs_ref}; $j++)
	    {
		if (   $subs_ref->[$j]->{type}
		    != $Email::List::Subscription::UNSUBSCRIPTION)
		{
		    next;
		}

		$matched
		    =  ($subs_ref->[$i]->{listname} eq $subs_ref->[$j]->{listname})
		    && ($subs_ref->[$i]->{subscriber} eq $subs_ref->[$j]->{subscriber});
		last unless !$matched;
	    }

	    if ($matched)
	    {
		# Found a pair - nuke 'em.
		splice(@{$subs_ref}, $i, 1);
		splice(@{$subs_ref}, $j - 1, 1);
		next;
	    }
	}

	# No match.
	$i++;
    }

    return 1;
}

1;
